@@ -69,6 +69,8 @@ struct body_status {
 
 typedef const struct body_status *body_status_t;
 
+typedef const char *hdr_t;
+
 /*--------------------------------------------------------------------*/
 
 enum sess_close {
@@ -87,35 +89,29 @@ enum {
 
 /*--------------------------------------------------------------------*/
 
-struct VSC_lck;
-struct VSC_main;
-struct VSC_main_wrk;
 struct ban;
 struct ban_proto;
 struct cli;
 struct http_conn;
+struct listen_sock;
 struct mempool;
 struct objcore;
 struct objhead;
 struct pool;
+struct req_step;
 struct sess;
 struct transport;
-struct worker;
-struct listen_sock;
 struct vcf;
-struct req_step;
+struct VSC_lck;
+struct VSC_main;
+struct VSC_main_wrk;
+struct worker;
+struct worker_priv;
 
 #define DIGEST_LEN		32
 
 /*--------------------------------------------------------------------*/
 
-typedef struct {
-	const char		*b;
-	const char		*e;
-} txt;
-
-/*--------------------------------------------------------------------*/
-
 struct lock { void *priv; };	// Opaque
 
 /*--------------------------------------------------------------------
@@ -182,13 +178,6 @@ struct vsl_log {
 
 /*--------------------------------------------------------------------*/
 
-struct vxid_pool {
-	uint32_t		next;
-	uint32_t		count;
-};
-
-/*--------------------------------------------------------------------*/
-
 VRBT_HEAD(vrt_privs, vrt_priv);
 
 /* Worker pool stuff -------------------------------------------------*/
@@ -229,28 +218,22 @@ enum task_prio {
 struct worker {
 	unsigned		magic;
 #define WORKER_MAGIC		0x6391adcf
+	int			strangelove;
+	struct worker_priv	*wpriv;
 	struct pool		*pool;
-	struct objhead		*nobjhead;
-	struct objcore		*nobjcore;
-	void			*nhashpriv;
 	struct VSC_main_wrk	*stats;
 	struct vsl_log		*vsl;		// borrowed from req/bo
 
 	struct pool_task	task[1];
 
 	vtim_real		lastused;
-	int			strangelove;
 
 	struct v1l		*v1l;
 
 	pthread_cond_t		cond;
 
-	struct vcl		*vcl;
-
 	struct ws		aws[1];
 
-	struct vxid_pool	vxid_pool;
-
 	unsigned		cur_method;
 	unsigned		seen_methods;
 	unsigned		handling;
@@ -606,40 +589,42 @@ void http_PutResponse(struct http *to, const char *proto, uint16_t status,
 void http_FilterReq(struct http *to, const struct http *fm, unsigned how);
 void HTTP_Encode(const struct http *fm, uint8_t *, unsigned len, unsigned how);
 int HTTP_Decode(struct http *to, const uint8_t *fm);
-void http_ForceHeader(struct http *to, const char *hdr, const char *val);
+void http_ForceHeader(struct http *to, hdr_t, const char *val);
 void http_PrintfHeader(struct http *to, const char *fmt, ...)
     v_printflike_(2, 3);
 void http_TimeHeader(struct http *to, const char *fmt, vtim_real now);
 void http_Proto(struct http *to);
-void http_SetHeader(struct http *to, const char *hdr);
-void http_SetH(struct http *to, unsigned n, const char *fm);
+void http_SetHeader(struct http *to, const char *header);
+void http_SetH(struct http *to, unsigned n, const char *header);
 void http_ForceField(struct http *to, unsigned n, const char *t);
 void HTTP_Setup(struct http *, struct ws *, struct vsl_log *, enum VSL_tag_e);
 void http_Teardown(struct http *ht);
-int http_GetHdr(const struct http *hp, const char *hdr, const char **ptr);
-int http_GetHdrToken(const struct http *hp, const char *hdr,
+int http_GetHdr(const struct http *hp, hdr_t, const char **ptr);
+int http_GetHdrToken(const struct http *hp, hdr_t,
     const char *token, const char **pb, const char **pe);
-int http_GetHdrField(const struct http *hp, const char *hdr,
+int http_GetHdrField(const struct http *hp, hdr_t,
     const char *field, const char **ptr);
-double http_GetHdrQ(const struct http *hp, const char *hdr, const char *field);
+double http_GetHdrQ(const struct http *hp, hdr_t, const char *field);
 ssize_t http_GetContentLength(const struct http *hp);
+ssize_t http_GetContentRange(const struct http *hp, ssize_t *lo, ssize_t *hi);
+const char * http_GetRange(const struct http *hp, ssize_t *lo, ssize_t *hi);
 uint16_t http_GetStatus(const struct http *hp);
 int http_IsStatus(const struct http *hp, int);
 void http_SetStatus(struct http *to, uint16_t status, const char *reason);
 const char *http_GetMethod(const struct http *hp);
-int http_HdrIs(const struct http *hp, const char *hdr, const char *val);
+int http_HdrIs(const struct http *hp, hdr_t, const char *val);
 void http_CopyHome(const struct http *hp);
-void http_Unset(struct http *hp, const char *hdr);
-unsigned http_CountHdr(const struct http *hp, const char *hdr);
-void http_CollectHdr(struct http *hp, const char *hdr);
-void http_CollectHdrSep(struct http *hp, const char *hdr, const char *sep);
+void http_Unset(struct http *hp, hdr_t);
+unsigned http_CountHdr(const struct http *hp, hdr_t);
+void http_CollectHdr(struct http *hp, hdr_t);
+void http_CollectHdrSep(struct http *hp, hdr_t, const char *sep);
 void http_VSL_log(const struct http *hp);
 void HTTP_Merge(struct worker *, struct objcore *, struct http *to);
 uint16_t HTTP_GetStatusPack(struct worker *, struct objcore *oc);
 int HTTP_IterHdrPack(struct worker *, struct objcore *, const char **);
 #define HTTP_FOREACH_PACK(wrk, oc, ptr) \
 	 for ((ptr) = NULL; HTTP_IterHdrPack(wrk, oc, &(ptr));)
-const char *HTTP_GetHdrPack(struct worker *, struct objcore *, const char *hdr);
+const char *HTTP_GetHdrPack(struct worker *, struct objcore *, hdr_t);
 enum sess_close http_DoConnection(struct http *hp, enum sess_close sc_close);
 int http_IsFiltered(const struct http *hp, unsigned u, unsigned how);
 
@@ -656,9 +641,39 @@ extern const char H__Status[];
 extern const char H__Proto[];
 extern const char H__Reason[];
 
+// rfc7233,l,1207,1208
+#define http_tok_eq(s1, s2)		(!vct_casecmp(s1, s2))
+#define http_tok_at(s1, s2, l)		(!vct_caselencmp(s1, s2, l))
+#define http_ctok_at(s, cs)		(!vct_caselencmp(s, cs, sizeof(cs) - 1))
+
+// rfc7230,l,1037,1038
+#define http_scheme_at(str, tok)	http_ctok_at(str, #tok "://")
+
+// rfc7230,l,1144,1144
+// rfc7231,l,1156,1158
+#define http_method_eq(str, tok)	(!strcmp(str, #tok))
+
+// rfc7230,l,1222,1222
+// rfc7230,l,2848,2848
+// rfc7231,l,3883,3885
+// rfc7234,l,1339,1340
+// rfc7234,l,1418,1419
+#define http_hdr_eq(s1, s2)		http_tok_eq(s1, s2)
+#define http_hdr_at(s1, s2, l)		http_tok_at(s1, s2, l)
+
+// rfc7230,l,1952,1952
+// rfc7231,l,604,604
+#define http_coding_eq(str, tok)	http_tok_eq(str, #tok)
+
+// rfc7231,l,1864,1864
+#define http_expect_eq(str, tok)	http_tok_eq(str, #tok)
+
+// rfc7233,l,1207,1208
+#define http_range_at(str, tok, l)	http_tok_at(str, #tok, l)
+
 /* cache_main.c */
 #define VXID(u) ((u) & VSL_IDENTMASK)
-uint32_t VXID_Get(struct worker *, uint32_t marker);
+uint32_t VXID_Get(const struct worker *, uint32_t marker);
 extern pthread_key_t witness_key;
 
 /* cache_lck.c */
@@ -758,15 +773,6 @@ VSLb_ts_busyobj(struct busyobj *bo, const char *event, vtim_real now)
 /* cache_vcl.c */
 const char *VCL_Name(const struct vcl *);
 
-/* cache_vrt.c */
-/*
- * These prototypes go here, because we do not want to pollute vrt.h
- * with va_list.  VCC never generates direct calls to them.
- * XXX: We should deprecate these (ref: STRANDS)
- */
-const char *VRT_String(struct ws *ws, const char *h, const char *p, va_list ap);
-char *VRT_StringList(char *d, unsigned dl, const char *p, va_list ap);
-
 /* cache_wrk.c */
 
 typedef void *bgthread_t(struct worker *, void *priv);
@@ -774,13 +780,10 @@ void WRK_BgThread(pthread_t *thr, const char *name, bgthread_t *func,
     void *priv);
 
 /* cache_ws.c */
-
 void WS_Init(struct ws *ws, const char *id, void *space, unsigned len);
 
 unsigned WS_ReserveSize(struct ws *, unsigned);
 unsigned WS_ReserveAll(struct ws *);
-unsigned WS_ReserveLumps(struct ws *ws, size_t sz);
-void WS_MarkOverflow(struct ws *ws);
 void WS_Release(struct ws *ws, unsigned bytes);
 void WS_ReleaseP(struct ws *ws, const char *ptr);
 void WS_Assert(const struct ws *ws);
@@ -788,21 +791,8 @@ void WS_Reset(struct ws *ws, uintptr_t);
 void *WS_Alloc(struct ws *ws, unsigned bytes);
 void *WS_Copy(struct ws *ws, const void *str, int len);
 uintptr_t WS_Snapshot(struct ws *ws);
-int WS_Overflowed(const struct ws *ws);
-const char *WS_Printf(struct ws *ws, const char *fmt, ...) v_printflike_(2, 3);
-int WS_Inside(const struct ws *, const void *, const void *);
-void WS_Assert_Allocated(const struct ws *ws, const void *ptr, ssize_t len);
-
-void WS_VSB_new(struct vsb *, struct ws *);
-char *WS_VSB_finish(struct vsb *, struct ws *, size_t *);
-
-/* REL_20210315 remove */
-static inline char*
-WS_Front(const struct ws *ws)
-{
-	AN(ws->r);
-	return ws->f;
-}
+int WS_Allocated(const struct ws *ws, const void *ptr, ssize_t len);
+unsigned WS_Dump(const struct ws *ws, char, size_t off, void *buf, size_t len);
 
 static inline void *
 WS_Reservation(const struct ws *ws)
@@ -822,6 +812,23 @@ WS_ReservationSize(const struct ws *ws)
 	return (ws->r - ws->f);
 }
 
+static inline unsigned
+WS_ReserveLumps(struct ws *ws, size_t sz)
+{
+
+	AN(sz);
+	return (WS_ReserveAll(ws) / sz);
+}
+
+/* cache_ws_common.c */
+void WS_MarkOverflow(struct ws *ws);
+int WS_Overflowed(const struct ws *ws);
+
+const char *WS_Printf(struct ws *ws, const char *fmt, ...) v_printflike_(2, 3);
+
+void WS_VSB_new(struct vsb *, struct ws *);
+char *WS_VSB_finish(struct vsb *, struct ws *, size_t *);
+
 /* cache_rfc2616.c */
 void RFC2616_Ttl(struct busyobj *, vtim_real now, vtim_real *t_origin,
     float *ttl, float *grace, float *keep);
@@ -830,36 +837,8 @@ int RFC2616_Do_Cond(const struct req *sp);
 void RFC2616_Weaken_Etag(struct http *hp);
 void RFC2616_Vary_AE(struct http *hp);
 
-#define Tcheck(t) do {						\
-		AN((t).b);					\
-		AN((t).e);					\
-		assert((t).b <= (t).e);				\
-	} while(0)
-
-/*
- * unsigned length of a txt
- */
-
-static inline unsigned
-Tlen(const txt t)
-{
-
-	Tcheck(t);
-	return ((unsigned)(t.e - t.b));
-}
-
 /*
  * We want to cache the most recent timestamp in wrk->lastused to avoid
  * extra timestamps in cache_pool.c.  Hide this detail with a macro
  */
 #define W_TIM_real(w) ((w)->lastused = VTIM_real())
-
-int PAN_dump_struct2(struct vsb *vsb, int block, const void *ptr,
-    const char *smagic, unsigned magic, const char *fmt, ...)
-    v_printflike_(6,7);
-
-#define PAN_dump_struct(vsb, ptr, magic, ...)		\
-    PAN_dump_struct2(vsb, 1, ptr, #magic, magic, __VA_ARGS__)
-
-#define PAN_dump_oneline(vsb, ptr, magic, ...)		\
-    PAN_dump_struct2(vsb, 0, ptr, #magic, magic, __VA_ARGS__)
