@@ -5,6 +5,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -40,7 +42,7 @@
 struct vbp_target;
 struct vrt_ctx;
 struct vrt_backend_probe;
-struct tcp_pool;
+struct conn_pool;
 
 /*--------------------------------------------------------------------
  * An instance of a backend from a VCL program.
@@ -53,7 +55,8 @@ struct backend {
 	unsigned		n_conn;
 
 	VTAILQ_ENTRY(backend)	list;
-	struct lock		mtx;
+
+	struct vrt_endpoint	*endpoint;
 
 	VRT_BACKEND_FIELDS()
 
@@ -65,7 +68,7 @@ struct backend {
 	struct vsc_seg		*vsc_seg;
 	struct VSC_vbe		*vsc;
 
-	struct tcp_pool		*tcp_pool;
+	struct conn_pool	*conn_pool;
 
 	VCL_BACKEND		director;
 
@@ -76,13 +79,10 @@ struct backend {
  * Prototypes
  */
 
-/* cache_backend_cfg.c */
-void VBE_SetHappy(const struct backend *, uint64_t);
-
 /* cache_backend_probe.c */
 void VBP_Update_Backend(struct vbp_target *vt);
 void VBP_Insert(struct backend *b, struct vrt_backend_probe const *p,
-    struct tcp_pool *);
+    struct conn_pool *);
 void VBP_Remove(struct backend *b);
 void VBP_Control(const struct backend *b, int stop);
 void VBP_Status(struct vsb *, const struct backend *, int details, int json);
