@@ -5,6 +5,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -34,19 +36,20 @@
 struct vcldir {
 	unsigned			magic;
 #define VCLDIR_MAGIC			0xbf726c7d
+	struct lock			dlck;
 	struct director			*dir;
 	struct vcl			*vcl;
 	const struct vdi_methods	*methods;
 	VTAILQ_ENTRY(vcldir)		list;
 	const struct vdi_ahealth	*admin_health;
-	double				health_changed;
+	vtim_real			health_changed;
 	char				*cli_name;
 };
 
 #define VBE_AHEALTH_LIST					\
 	VBE_AHEALTH(healthy,	HEALTHY,	1)		\
 	VBE_AHEALTH(sick,	SICK,		0)		\
-	VBE_AHEALTH(probe,	PROBE,		-1)		\
+	VBE_AHEALTH(auto,	AUTO,		-1)		\
 	VBE_AHEALTH(deleted,	DELETED,	0)
 
 #define VBE_AHEALTH(l,u,h) extern const struct vdi_ahealth * const VDI_AH_##u;
