@@ -47,12 +47,43 @@
 #  include "VSC_main.h"
 #endif
 
-/* -------------------------------------------------------------------*/
+/*--------------------------------------------------------------------*/
 
 struct vfp;
+struct vdp;
 struct cli_proto;
 struct poolparam;
 
+/*--------------------------------------------------------------------*/
+
+typedef enum req_fsm_nxt req_state_f(struct worker *, struct req *);
+struct req_step {
+	const char	*name;
+	req_state_f	*func;
+};
+
+extern const struct req_step R_STP_TRANSPORT[1];
+extern const struct req_step R_STP_RECV[1];
+
+struct vxid_pool {
+	uint32_t		next;
+	uint32_t		count;
+};
+
+/*--------------------------------------------------------------------
+ * Private part of worker threads
+ */
+
+struct worker_priv {
+	unsigned		magic;
+#define WORKER_PRIV_MAGIC	0x3047db99
+	struct objhead		*nobjhead;
+	struct objcore		*nobjcore;
+	void			*nhashpriv;
+	struct vxid_pool	vxid_pool[1];
+	struct vcl		*vcl;
+};
+
 /*--------------------------------------------------------------------
  * HTTP Protocol connection structure
  *
@@ -133,10 +164,6 @@ void VCA_Shutdown(void);
 void VBE_InitCfg(void);
 void VBE_Poll(void);
 
-/* cache_backend_tcp.c */
-void VTP_Init(void);
-
-/* cache_backend_poll.c */
 void VBP_Init(void);
 
 /* cache_ban.c */
@@ -150,7 +177,7 @@ void BAN_RefBan(struct objcore *oc, struct ban *);
 vtim_real BAN_Time(const struct ban *ban);
 
 /* cache_busyobj.c */
-struct busyobj *VBO_GetBusyObj(struct worker *, const struct req *);
+struct busyobj *VBO_GetBusyObj(const struct worker *, const struct req *);
 void VBO_ReleaseBusyObj(struct worker *wrk, struct busyobj **busyobj);
 
 /* cache_director.c */
@@ -162,9 +189,22 @@ void VDI_Panic(const struct director *, struct vsb *, const char *nm);
 void VDI_Event(const struct director *d, enum vcl_event_e ev);
 void VDI_Init(void);
 
+/* cache_deliver_proc.c */
+void VDP_Init(struct vdp_ctx *vdx, struct worker *wrk, struct vsl_log *vsl,
+    struct req *req);
+uint64_t VDP_Close(struct vdp_ctx *);
+void VDP_Panic(struct vsb *vsb, const struct vdp_ctx *vdc);
+int VDP_Push(struct vdp_ctx *, struct ws *, const struct vdp *, void *priv);
+int VDP_DeliverObj(struct vdp_ctx *vdc, struct objcore *oc);
+extern const struct vdp VDP_gunzip;
+extern const struct vdp VDP_esi;
+extern const struct vdp VDP_range;
+
+
 /* cache_exp.c */
 vtim_real EXP_Ttl(const struct req *, const struct objcore *);
 vtim_real EXP_Ttl_grace(const struct req *, const struct objcore *oc);
+void EXP_RefNewObjcore(struct objcore *);
 void EXP_Insert(struct worker *wrk, struct objcore *oc);
 void EXP_Remove(struct objcore *);
 
@@ -221,14 +261,6 @@ void CLI_Init(void);
 void CLI_Run(void);
 void CLI_AddFuncs(struct cli_proto *p);
 
-/* cache_deliver_proc.c */
-void VDP_close(struct req *req);
-int VDP_DeliverObj(struct req *req);
-
-extern const struct vdp VDP_gunzip;
-extern const struct vdp VDP_esi;
-extern const struct vdp VDP_range;
-
 /* cache_expire.c */
 void EXP_Init(void);
 void EXP_Shutdown(void);
@@ -242,16 +274,16 @@ enum vbf_fetch_mode_e {
 void VBF_Fetch(struct worker *wrk, struct req *req,
     struct objcore *oc, struct objcore *oldoc, enum vbf_fetch_mode_e);
 const char *VBF_Get_Filter_List(struct busyobj *);
-void Bereq_Rollback(struct busyobj *);
+void Bereq_Rollback(VRT_CTX);
 
 /* cache_fetch_proc.c */
 void VFP_Init(void);
-enum vfp_status VFP_GetStorage(struct vfp_ctx *, ssize_t *sz, uint8_t **ptr);
-void VFP_Extend(const struct vfp_ctx *, ssize_t sz);
 struct vfp_entry *VFP_Push(struct vfp_ctx *, const struct vfp *);
+enum vfp_status VFP_GetStorage(struct vfp_ctx *, ssize_t *sz, uint8_t **ptr);
+void VFP_Extend(const struct vfp_ctx *, ssize_t sz, enum vfp_status);
 void VFP_Setup(struct vfp_ctx *vc, struct worker *wrk);
-int VFP_Open(struct vfp_ctx *bo);
-void VFP_Close(struct vfp_ctx *bo);
+int VFP_Open(struct vfp_ctx *);
+uint64_t VFP_Close(struct vfp_ctx *);
 
 extern const struct vfp VFP_gunzip;
 extern const struct vfp VFP_gzip;
@@ -277,6 +309,8 @@ void THR_SetBusyobj(const struct busyobj *);
 struct busyobj * THR_GetBusyobj(void);
 void THR_SetRequest(const struct req *);
 struct req * THR_GetRequest(void);
+void THR_SetWorker(const struct worker *);
+struct worker * THR_GetWorker(void);
 void THR_Init(void);
 
 /* cache_lck.c */
@@ -295,13 +329,12 @@ void ObjInit(void);
 struct objcore * ObjNew(const struct worker *);
 void ObjDestroy(const struct worker *, struct objcore **);
 int ObjGetSpace(struct worker *, struct objcore *, ssize_t *sz, uint8_t **ptr);
-void ObjExtend(struct worker *, struct objcore *, ssize_t l);
+void ObjExtend(struct worker *, struct objcore *, ssize_t l, int final);
 uint64_t ObjWaitExtend(const struct worker *, const struct objcore *,
     uint64_t l);
 void ObjSetState(struct worker *, const struct objcore *,
     enum boc_state_e next);
 void ObjWaitState(const struct objcore *, enum boc_state_e want);
-void ObjTrimStore(struct worker *, struct objcore *);
 void ObjTouch(struct worker *, struct objcore *, vtim_real now);
 void ObjFreeObj(struct worker *, struct objcore *);
 void ObjSlim(struct worker *, struct objcore *);
@@ -334,7 +367,22 @@ void ObjUnsubscribeEvents(uintptr_t *);
 
 /* cache_panic.c */
 void PAN_Init(void);
-int PAN_already(struct vsb *, const void *);
+int PAN__DumpStruct(struct vsb *vsb, int block, int track, const void *ptr,
+    const char *smagic, unsigned magic, const char *fmt, ...)
+    v_printflike_(7,8);
+
+#define PAN_dump_struct(vsb, ptr, magic, ...)		\
+    PAN__DumpStruct(vsb, 1, 1, ptr, #magic, magic, __VA_ARGS__)
+
+#define PAN_dump_oneline(vsb, ptr, magic, ...)		\
+    PAN__DumpStruct(vsb, 0, 1, ptr, #magic, magic, __VA_ARGS__)
+
+#define PAN_dump_once(vsb, ptr, magic, ...)		\
+    PAN__DumpStruct(vsb, 1, 0, ptr, #magic, magic, __VA_ARGS__)
+
+#define PAN_dump_once_oneline(vsb, ptr, magic, ...)		\
+    PAN__DumpStruct(vsb, 0, 0, ptr, #magic, magic, __VA_ARGS__)
+
 const char *sess_close_2str(enum sess_close sc, int want_desc);
 
 /* cache_pool.c */
@@ -346,20 +394,23 @@ void Pool_Sumstat(const struct worker *w);
 int Pool_TrySumstat(const struct worker *wrk);
 void Pool_PurgeStat(unsigned nobj);
 int Pool_Task_Any(struct pool_task *task, enum task_prio prio);
+void pan_pool(struct vsb *);
+
+/* cache_range.c */
+int VRG_CheckBo(struct busyobj *);
 
 /* cache_req.c */
 struct req *Req_New(const struct worker *, struct sess *);
 void Req_Release(struct req *);
-void Req_Rollback(struct req *req);
+void Req_Rollback(VRT_CTX);
 void Req_Cleanup(struct sess *sp, struct worker *wrk, struct req *req);
 void Req_Fail(struct req *req, enum sess_close reason);
 void Req_AcctLogCharge(struct VSC_main_wrk *, struct req *);
+void Req_LogHit(struct worker *, struct req *, struct objcore *, intmax_t);
 
 /* cache_req_body.c */
 int VRB_Ignore(struct req *);
 ssize_t VRB_Cache(struct req *, ssize_t maxsize);
-ssize_t VRB_Iterate(struct worker *, struct vsl_log *, struct req *,
-    objiterate_f *func, void *priv);
 void VRB_Free(struct req *);
 
 /* cache_req_fsm.c [CNT] */
@@ -385,14 +436,14 @@ void SES_Rel(struct sess *sp);
 
 const char * HTC_Status(enum htc_status_e);
 void HTC_RxInit(struct http_conn *htc, struct ws *ws);
-void HTC_RxPipeline(struct http_conn *htc, void *);
+void HTC_RxPipeline(struct http_conn *htc, char *);
 enum htc_status_e HTC_RxStuff(struct http_conn *, htc_complete_f *,
     vtim_real *t1, vtim_real *t2, vtim_real ti, vtim_real tn, vtim_dur td,
     int maxbytes);
 
 #define SESS_ATTR(UP, low, typ, len)					\
 	int SES_Set_##low(const struct sess *sp, const typ *src);	\
-	int SES_Reserve_##low(struct sess *sp, typ **dst);
+	int SES_Reserve_##low(struct sess *sp, typ **dst, ssize_t *sz);
 #include "tbl/sess_attr.h"
 int SES_Set_String_Attr(struct sess *sp, enum sess_attr a, const char *src);
 
@@ -405,6 +456,13 @@ void VSL_ChgId(struct vsl_log *vsl, const char *typ, const char *why,
 void VSL_End(struct vsl_log *vsl);
 void VSL_Flush(struct vsl_log *, int overflow);
 
+/* cache_conn_pool.c */
+struct conn_pool;
+void VCP_Init(void);
+void VCP_Panic(struct vsb *, struct conn_pool *);
+
+/* cache_backend_poll.c */
+
 /* cache_vary.c */
 int VRY_Create(struct busyobj *bo, struct vsb **psb);
 int VRY_Match(struct req *, const uint8_t *vary);
@@ -414,23 +472,13 @@ enum vry_finish_flag { KEEP, DISCARD };
 void VRY_Finish(struct req *req, enum vry_finish_flag);
 
 /* cache_vcl.c */
-VCL_BACKEND VCL_DefaultDirector(const struct vcl *);
-const struct vrt_backend_probe *VCL_DefaultProbe(const struct vcl *);
-void VCL_Init(void);
-void VCL_Panic(struct vsb *, const char *nm, const struct vcl *);
-void VCL_Poll(void);
-void VCL_Ref(struct vcl *);
-void VCL_Refresh(struct vcl **);
-void VCL_Recache(struct worker *, struct vcl **);
-void VCL_Rel(struct vcl **);
-void VCL_TaskEnter(struct vrt_privs *);
-void VCL_TaskLeave(struct vrt_privs *);
-const char *VCL_Return_Name(unsigned);
-const char *VCL_Method_Name(unsigned);
 void VCL_Bo2Ctx(struct vrt_ctx *, struct busyobj *);
 void VCL_Req2Ctx(struct vrt_ctx *, struct req *);
 struct vrt_ctx *VCL_Get_CliCtx(int);
 struct vsb *VCL_Rel_CliCtx(struct vrt_ctx **);
+void VCL_Panic(struct vsb *, const char *nm, const struct vcl *);
+void VCL_Poll(void);
+void VCL_Init(void);
 
 #define VCL_MET_MAC(l,u,t,b) \
     void VCL_##l##_method(struct vcl *, struct worker *, struct req *, \
@@ -442,9 +490,6 @@ typedef int vcl_be_func(struct cli *, struct director *, void *);
 
 int VCL_IterDirector(struct cli *, const char *, vcl_be_func *, void *);
 
-/* cache_vcl_vrt.c */
-void VCL_VRT_Init(void);
-
 /* cache_vrt.c */
 void pan_privs(struct vsb *, const struct vrt_privs *);
 
@@ -452,18 +497,50 @@ void pan_privs(struct vsb *, const struct vrt_privs *);
 int VCL_StackVFP(struct vfp_ctx *, const struct vcl *, const char *);
 int VCL_StackVDP(struct req *, const struct vcl *, const char *);
 const char *resp_Get_Filter_List(struct req *req);
+void VCL_VRT_Init(void);
+
+/* cache_vrt_vcl.c */
+const char *VCL_Return_Name(unsigned);
+const char *VCL_Method_Name(unsigned);
+void VCL_Refresh(struct vcl **);
+void VCL_Recache(const struct worker *, struct vcl **);
+void VCL_Ref(struct vcl *);
+void VCL_Rel(struct vcl **);
+VCL_BACKEND VCL_DefaultDirector(const struct vcl *);
+const struct vrt_backend_probe *VCL_DefaultProbe(const struct vcl *);
 
 /* cache_vrt_priv.c */
 extern struct vrt_privs cli_task_privs[1];
+void VCL_TaskEnter(struct vrt_privs *);
+void VCL_TaskLeave(VRT_CTX, struct vrt_privs *);
 
 /* cache_vrt_vmod.c */
 void VMOD_Init(void);
 void VMOD_Panic(struct vsb *);
 
+#if defined(ENABLE_COVERAGE) || defined(ENABLE_SANITIZER)
+#  define DONT_DLCLOSE_VMODS
+#endif
+
 /* cache_wrk.c */
 void WRK_Init(void);
+void WRK_AddStat(const struct worker *);
 
 /* cache_ws.c */
+void WS_Panic(struct vsb *, const struct ws *);
+static inline int
+WS_IsReserved(const struct ws *ws)
+{
+
+	return (ws->r != NULL);
+}
+
+void *WS_AtOffset(const struct ws *ws, unsigned off, unsigned len);
+unsigned WS_ReservationOffset(const struct ws *ws);
+unsigned WS_ReqPipeline(struct ws *, const void *b, const void *e);
+
+/* cache_ws_common.c */
+void WS_Id(const struct ws *ws, char *id);
 void WS_Rollback(struct ws *, uintptr_t);
 
 /* http1/cache_http1_pipe.c */
@@ -482,6 +559,11 @@ void STV_BanExport(const uint8_t *banlist, unsigned len);
 int STV_NewObject(struct worker *, struct objcore *,
     const struct stevedore *, unsigned len);
 
+struct stv_buffer;
+struct stv_buffer *STV_AllocBuf(struct worker *wrk, const struct stevedore *stv,
+    size_t size);
+void STV_FreeBuf(struct worker *wrk, struct stv_buffer **pstvbuf);
+void *STV_GetBufPtr(struct stv_buffer *stvbuf, size_t *psize);
 
 #if WITH_PERSISTENT_STORAGE
 /* storage_persistent.c */
