@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2015 Varnish Software AS
+ * Copyright (c) 2006-2021 Varnish Software AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -53,6 +53,72 @@
  * Whenever something is deleted or changed in a way which is not
  * binary/load-time compatible, increment MAJOR version
  *
+ * 14.0 (2021-09-15)
+ *	VIN_n_Arg() no directly returns the directory name.
+ *	VSB_new() and VSB_delete() removed
+ *	VCL_STRINGLIST, vrt_magic_string_end removed
+ *	VRT_String(), VRT_StringList(), VRT_CollectString() removed
+ *	VRT_CollectStrands() renamed to VRT_STRANDS_string()
+ *	The following functions changed to take `const char *, STRANDS`:
+ *	- VRT_l_client_identity()
+ *	- VRT_l_req_method()
+ *	- VRT_l_req_url()
+ *	- VRT_l_req_proto()
+ *	- VRT_l_bereq_method()
+ *	- VRT_l_bereq_url()
+ *	- VRT_l_bereq_proto()
+ *	- VRT_l_beresp_body()
+ *	- VRT_l_beresp_proto()
+ *	- VRT_l_beresp_reason()
+ *	- VRT_l_beresp_storage_hint()
+ *	- VRT_l_beresp_filters()
+ *	- VRT_l_resp_body()
+ *	- VRT_l_resp_proto()
+ *	- VRT_l_resp_reason()
+ *	- VRT_l_resp_filters()
+ *	- VRT_SetHdr()
+ *	VRT_UnsetHdr() added
+ *	vrt_magic_string_unset removed (use VRT_UnsetHdr() instead)
+ *	VNUMpfx() removed, SF_Parse_{Integer|Decimal|Number} added
+ *	vrt_null_strands added
+ *	vrt_null_blob added
+ *	VRT_NULL_BLOB_TYPE added as the .type of vrt_null_blob
+ *	VRT_blob() changed to return vrt_null_blob for
+ *	    len == 0 or src == NULL arguments
+ *	[cache.h] WS_Front() removed
+ *	[cache.h] WS_Inside() removed
+ *	[cache.h] WS_Assert_Allocated() removed
+ *	[cache.h] WS_Allocated() added
+ *	[cache.h] WS_Dump() added
+ * 13.0 (2021-03-15)
+ *	Move VRT_synth_page() to deprecated status
+ *	Add VRT_synth_strands() and VRT_synth_blob()
+ *	struct vrt_type now produced by generate.py
+ *	VRT_acl_log() moved to VPI_acl_log()
+ *	VRT_Endpoint_Clone() added.
+ *	Calling convention for VDP implementation changed
+ *	VRT_ValidHdr() added.
+ *	struct vmod_priv_methods added
+ *	struct vmod_priv free member replaced with methods
+ *	VRT_CTX_Assert() added
+ *	VRT_ban_string() signature changed
+ *	VRT_priv_task_get() added
+ *	VRT_priv_top_get() added
+ *	VRT_re_init removed
+ *	VRT_re_fini removed
+ *	VRT_re_match signature changed
+ *	VRT_regsub signature changed
+ *	VRT_call() added
+ *	VRT_check_call() added
+ *	VRT_handled() added
+ * 12.0 (2020-09-15)
+ *	Added VRT_DirectorResolve()
+ *	Added VCL_STRING VRT_BLOB_string(VRT_CTX, VCL_BLOB)
+ *	[cache.h] WS_Reserve() removed
+ *	[cache.h] WS_Printf() changed
+ *	[cache.h] WS_ReservationSize() added
+ *	[cache.h] WS_Front() deprecated
+ *	[cache.h] WS_Reservation() added
  * 11.0 (2020-03-16)
  *	Changed type of vsa_suckaddr_len from int to size_t
  *	New prefix_{ptr|len} fields in vrt_backend
@@ -159,7 +225,7 @@
  *	vrt_acl type added
  */
 
-#define VRT_MAJOR_VERSION	11U
+#define VRT_MAJOR_VERSION	14U
 
 #define VRT_MINOR_VERSION	0U
 
@@ -168,38 +234,41 @@
 #include <stddef.h>		// NULL, size_t
 #include <stdint.h>		// [u]int%d_t
 
-struct VCL_conf;
 struct busyobj;
 struct director;
 struct http;
+struct lock;
 struct req;
 struct stevedore;
 struct suckaddr;
 struct vcl;
+struct vcldir;
+struct VCL_conf;
+struct vcl_sub;
 struct vmod;
 struct vmod_priv;
 struct vrt_acl;
 struct vsb;
-struct vsc_seg;
-struct vsmw_cluster;
-struct vsl_log;
-struct ws;
 struct VSC_main;
+struct vsc_seg;
+struct vsl_log;
+struct vsmw_cluster;
+struct ws;
 
-/*
+/***********************************************************************
  * VCL_STRANDS:
  *
- * An argc+argv type of data structure where n indicates the number of strings
- * in the p array. Individual components of a strands may be null.
+ * An argc+argv type of data structure where n indicates the number of
+ * strings in the p array. Individual components of a strands may be null.
  *
- * A STRANDS allows you to work on a strings concatenation with the option to
- * collect it into a single STRING, or if possible work directly on individual
- * parts.
+ * A STRANDS allows you to work on a strings concatenation with the
+ * option to collect it into a single STRING, or if possible work
+ * directly on individual parts.
  *
- * The memory management is very strict: a VMOD function receiving a STRANDS
- * argument should keep no reference after the function returns. Retention of
- * a STRANDS further in the ongoing task is undefined behavior and may result
- * in a panic or data corruption.
+ * The memory management is very strict: a VMOD function receiving a
+ * STRANDS argument should keep no reference after the function returns.
+ * Retention of a STRANDS further in the ongoing task is undefined
+ * behavior and may result in a panic or data corruption.
  */
 
 struct strands {
@@ -207,23 +276,24 @@ struct strands {
 	const char	**p;
 };
 
-struct strands * VRT_AllocStrandsWS(struct ws *, int);
-
-
 /*
+ * A VCL_STRANDS return value must never be NULL. Use this instead
+ */
+extern const struct strands *vrt_null_strands;
+
+/***********************************************************************
  * VCL_BLOB:
  *
- * Opaque, immutable data (pointer + length), minimum lifetime is the VCL task.
+ * Opaque, immutable data (pointer + length), minimum lifetime is the
+ * VCL task.
  *
- * Type (optional) is owned by the creator of the blob. blob consumers may use
- * it for checks, but should not assert on it.
+ * Type (optional) is owned by the creator of the blob. blob consumers
+ * may use it for checks, but should not assert on it.
  *
- * The data behind the blob pointer is assumed to be immutable for the blob's
- * lifetime.
+ * The data behind the blob pointer is assumed to be immutable for the
+ * blob's lifetime.
  *
- * Memory management is either implicit or up to the vmod:
- *
- * - memory for shortlived blobs should come from the respective workspace
+ * - memory for shortlived blobs can be put on the tasks workspace
  *
  * - management of memory for longer lived blobs is up to the vmod
  *   (in which case the blob will probably be embedded in an object or
@@ -236,10 +306,13 @@ struct vrt_blob {
 	const void	*blob;
 };
 
+#define VRT_NULL_BLOB_TYPE 0xfade4fa0
+extern const struct vrt_blob *vrt_null_blob;
+
 /***********************************************************************
  * This is the central definition of the mapping from VCL types to
  * C-types.  The python scripts read these from here.
- * (alphabetic order)
+ * (keep alphabetic order)
  */
 
 typedef const struct vrt_acl *			VCL_ACL;
@@ -257,30 +330,18 @@ typedef int64_t					VCL_INT;
 typedef const struct suckaddr *			VCL_IP;
 typedef const struct vrt_backend_probe *	VCL_PROBE;
 typedef double					VCL_REAL;
+typedef const struct vre *			VCL_REGEX;
 typedef const struct stevedore *		VCL_STEVEDORE;
 typedef const struct strands *			VCL_STRANDS;
 typedef const char *				VCL_STRING;
+typedef const struct vcl_sub *			VCL_SUB;
 typedef vtim_real				VCL_TIME;
 typedef struct vcl *				VCL_VCL;
 typedef void					VCL_VOID;
 
-struct vrt_type {
-	unsigned			magic;
-#define VRT_TYPE_MAGIC			0xa943bc32
-	const char			*lname;
-	const char			*uname;
-	const char			*ctype;
-	size_t				szof;
-};
-
-enum lbody_e {
-	LBODY_SET,
-	LBODY_ADD,
-};
-
 /***********************************************************************
- * This is the composite argument we pass to compiled VCL and VRT
- * functions.
+ * This is the composite "context" argument for compiled VCL, VRT and
+ * VMOD functions.
  */
 
 struct vrt_ctx {
@@ -288,9 +349,9 @@ struct vrt_ctx {
 #define VRT_CTX_MAGIC			0x6bb8f0db
 
 	unsigned			syntax;
+	unsigned			vclver;
 	unsigned			method;
 	unsigned			*handling;
-	unsigned			vclver;
 
 	/*
 	 * msg is for error messages and exists only for
@@ -321,12 +382,30 @@ struct vrt_ctx {
 	 *    synth+error:	struct vsb *
 	 */
 	void				*specific;
+	/* if present, vbitmap of called subs */
+	void				*called;
 };
 
 #define VRT_CTX		const struct vrt_ctx *ctx
+void VRT_CTX_Assert(VRT_CTX);
+
+enum vcl_func_call_e {
+	VSUB_STATIC,	// VCL "call" action, only allowed from VCC
+	VSUB_DYNAMIC,	// VRT_call()
+	VSUB_CHECK	// VRT_check_call()
+};
+
+enum vcl_func_fail_e {
+	VSUB_E_OK,
+	VSUB_E_RECURSE, // call would recurse
+	VSUB_E_METHOD	// can not be called from this method
+};
+
+typedef void vcl_func_f(VRT_CTX, enum vcl_func_call_e, enum vcl_func_fail_e *);
 
 /***********************************************************************
  * This is the interface structure to a compiled VMOD
+ * (produced by vmodtool.py)
  */
 
 struct vmod_data {
@@ -345,7 +424,7 @@ struct vmod_data {
 };
 
 /***********************************************************************
- * Enum for events sent to compiled VCL and from there to Vmods
+ * VCL events sent to VMODs
  */
 
 enum vcl_event_e {
@@ -355,10 +434,69 @@ enum vcl_event_e {
 	VCL_EVENT_DISCARD,
 };
 
-/***********************************************************************/
+typedef int vmod_event_f(VRT_CTX, struct vmod_priv *, enum vcl_event_e);
 
-extern const void * const vrt_magic_string_end;
-extern const void * const vrt_magic_string_unset;
+/***********************************************************************
+ * Utility functions operating on VCL_types
+ * (alphabetic by type-ish)
+ */
+
+/* VCL_ACL */
+int VRT_acl_match(VRT_CTX, VCL_ACL, VCL_IP);
+
+/* VCL_BACKEND */
+VCL_BACKEND VRT_DirectorResolve(VRT_CTX, VCL_BACKEND);
+
+/* VCL_BLOB */
+VCL_BLOB VRT_blob(VRT_CTX, const char *, const void *, size_t, unsigned);
+
+/* VCL_INT */
+int VRT_INT_is_valid(VCL_INT);
+
+/* VCL_IP */
+int VRT_VSA_GetPtr(VRT_CTX, VCL_IP sua, const unsigned char ** dst);
+VCL_BOOL VRT_ipcmp(VRT_CTX, VCL_IP, VCL_IP);
+void VRT_Format_Proxy(struct vsb *, VCL_INT, VCL_IP, VCL_IP, VCL_STRING);
+
+/* VCL_REAL */
+int VRT_REAL_is_valid(VCL_REAL);
+
+/* VCL_REGEX */
+VCL_BOOL VRT_re_match(VRT_CTX, VCL_STRING, VCL_REGEX);
+VCL_STRING VRT_regsub(VRT_CTX, int all, VCL_STRING, VCL_REGEX, VCL_STRING);
+
+/* VCL_STEVEDORE */
+VCL_STEVEDORE VRT_stevedore(const char *nm);
+
+/* VCL_STRANDS */
+struct strands * VRT_AllocStrandsWS(struct ws *, int);
+int VRT_CompareStrands(VCL_STRANDS a, VCL_STRANDS b);
+VCL_BOOL VRT_Strands2Bool(VCL_STRANDS);
+uint32_t VRT_HashStrands32(VCL_STRANDS);
+char *VRT_Strands(char *, size_t, VCL_STRANDS);
+VCL_STRING VRT_StrandsWS(struct ws *, const char *, VCL_STRANDS);
+VCL_STRING VRT_UpperLowerStrands(VRT_CTX, VCL_STRANDS s, int up);
+
+/* VCL_SUB */
+VCL_STRING VRT_check_call(VRT_CTX, VCL_SUB);
+VCL_VOID VRT_call(VRT_CTX, VCL_SUB);
+
+/* Functions to turn types into canonical strings */
+
+VCL_STRING VRT_BACKEND_string(VCL_BACKEND);
+VCL_STRING VRT_BOOL_string(VCL_BOOL);
+VCL_STRING VRT_BLOB_string(VRT_CTX, VCL_BLOB);
+VCL_STRING VRT_CollectString(VRT_CTX, const char *p, ...);
+VCL_STRING VRT_INT_string(VRT_CTX, VCL_INT);
+VCL_STRING VRT_IP_string(VRT_CTX, VCL_IP);
+VCL_STRING VRT_REAL_string(VRT_CTX, VCL_REAL);
+VCL_STRING VRT_STEVEDORE_string(VCL_STEVEDORE);
+VCL_STRING VRT_STRANDS_string(VRT_CTX, VCL_STRANDS);
+VCL_STRING VRT_TIME_string(VRT_CTX, VCL_TIME);
+
+/* historical */
+int VRT_strcmp(const char *s1, const char *s2);
+void VRT_memmove(void *dst, const void *src, unsigned len);
 
 /***********************************************************************
  * We want the VCC to spit this structs out as const, but when VMODs
@@ -370,28 +508,27 @@ extern const void * const vrt_magic_string_unset;
  * One of those things...
  */
 
+struct vrt_endpoint {
+	unsigned			magic;
+#define VRT_ENDPOINT_MAGIC		0xcc419347
+	VCL_IP				ipv4;
+	VCL_IP				ipv6;
+	const char			*uds_path;
+	const struct vrt_blob		*preamble;
+};
+
 #define VRT_BACKEND_FIELDS(rigid)				\
 	rigid char			*vcl_name;		\
-	rigid char			*ipv4_addr;		\
-	rigid char			*ipv6_addr;		\
-	rigid char			*port;			\
-	rigid char			*path;			\
 	rigid char			*hosthdr;		\
 	vtim_dur			connect_timeout;	\
 	vtim_dur			first_byte_timeout;	\
 	vtim_dur			between_bytes_timeout;	\
 	unsigned			max_connections;	\
-	unsigned			proxy_header;		\
-	void				*prefix_ptr;		\
-	unsigned			prefix_len;
+	unsigned			proxy_header;
 
 #define VRT_BACKEND_HANDLE()			\
 	do {					\
 		DA(vcl_name);			\
-		DA(ipv4_addr);			\
-		DA(ipv6_addr);			\
-		DA(port);			\
-		DA(path);			\
 		DA(hosthdr);			\
 		DN(connect_timeout);		\
 		DN(first_byte_timeout);		\
@@ -403,9 +540,8 @@ extern const void * const vrt_magic_string_unset;
 struct vrt_backend {
 	unsigned			magic;
 #define VRT_BACKEND_MAGIC		0x4799ce6c
+	const struct vrt_endpoint	*endpoint;
 	VRT_BACKEND_FIELDS(const)
-	VCL_IP				ipv4_suckaddr;
-	VCL_IP				ipv6_suckaddr;
 	VCL_PROBE			probe;
 };
 
@@ -435,21 +571,14 @@ struct vrt_backend_probe {
 	VRT_BACKEND_PROBE_FIELDS(const)
 };
 
+/* Backend related */
+VCL_BACKEND VRT_new_backend(VRT_CTX, const struct vrt_backend *);
+VCL_BACKEND VRT_new_backend_clustered(VRT_CTX,
+    struct vsmw_cluster *, const struct vrt_backend *);
+size_t VRT_backend_vsm_need(VRT_CTX);
+void VRT_delete_backend(VRT_CTX, VCL_BACKEND *);
+struct vrt_endpoint *VRT_Endpoint_Clone(const struct vrt_endpoint *vep);
 
-/***********************************************************************
- * Implementation details of ACLs
- */
-
-void VRT_acl_log(VRT_CTX, const char *);
-int VRT_acl_match(VRT_CTX, VCL_ACL, VCL_IP);
-
-/***********************************************************************
- * Compile time regexp
- */
-
-void VRT_re_init(void **, const char *);
-void VRT_re_fini(void *);
-int VRT_re_match(VRT_CTX, const char *, void *);
 
 /***********************************************************************
  * Getting hold of the various struct http
@@ -476,44 +605,35 @@ VCL_STRING VRT_GetHdr(VRT_CTX, VCL_HEADER);
  * req related
  */
 
+enum lbody_e {
+	LBODY_SET,
+	LBODY_ADD,
+};
+
 VCL_BYTES VRT_CacheReqBody(VRT_CTX, VCL_BYTES maxsize);
 
-/* Regexp related */
-
-const char *VRT_regsub(VRT_CTX, int all, const char *, void *, const char *);
-VCL_VOID VRT_ban_string(VRT_CTX, VCL_STRING);
+VCL_STRING VRT_ban_string(VRT_CTX, VCL_STRING);
 VCL_INT VRT_purge(VRT_CTX, VCL_DURATION, VCL_DURATION, VCL_DURATION);
 VCL_VOID VRT_synth(VRT_CTX, VCL_INT, VCL_STRING);
 VCL_VOID VRT_hit_for_pass(VRT_CTX, VCL_DURATION);
 
-VCL_VOID VRT_SetHdr(VRT_CTX, VCL_HEADER, const char *, ...);
+VCL_BOOL VRT_ValidHdr(VRT_CTX, VCL_STRANDS);
+VCL_VOID VRT_UnsetHdr(VRT_CTX, VCL_HEADER);
+VCL_VOID VRT_SetHdr(VRT_CTX, VCL_HEADER, const char *pfx, VCL_STRANDS);
 VCL_VOID VRT_handling(VRT_CTX, unsigned hand);
+unsigned VRT_handled(VRT_CTX);
 VCL_VOID VRT_fail(VRT_CTX, const char *fmt, ...) v_printflike_(2,3);
 VCL_VOID VRT_hashdata(VRT_CTX, VCL_STRANDS);
 
-/* Simple stuff */
-int VRT_strcmp(const char *s1, const char *s2);
-void VRT_memmove(void *dst, const void *src, unsigned len);
-VCL_BOOL VRT_ipcmp(VRT_CTX, VCL_IP, VCL_IP);
-VCL_BLOB VRT_blob(VRT_CTX, const char *, const void *, size_t, unsigned);
-
 VCL_VOID VRT_Rollback(VRT_CTX, VCL_HTTP);
 
 /* Synthetic pages */
-VCL_VOID VRT_synth_page(VRT_CTX, VCL_STRANDS);
+VCL_VOID VRT_synth_strands(VRT_CTX, VCL_STRANDS);
+VCL_VOID VRT_synth_blob(VRT_CTX, VCL_BLOB);
 
-/* Backend related */
-VCL_BACKEND VRT_new_backend(VRT_CTX, const struct vrt_backend *);
-VCL_BACKEND VRT_new_backend_clustered(VRT_CTX,
-    struct vsmw_cluster *, const struct vrt_backend *);
-size_t VRT_backend_vsm_need(VRT_CTX);
-void VRT_delete_backend(VRT_CTX, VCL_BACKEND *);
-
-/* VSM related */
-struct vsmw_cluster *VRT_VSM_Cluster_New(VRT_CTX, size_t);
-void VRT_VSM_Cluster_Destroy(VRT_CTX, struct vsmw_cluster **);
-
-/* VDI - Director API */
+/***********************************************************************
+ * VDI - Director API
+ */
 typedef VCL_BOOL vdi_healthy_f(VRT_CTX, VCL_BACKEND, VCL_TIME *);
 typedef VCL_BACKEND vdi_resolve_f(VRT_CTX, VCL_BACKEND);
 typedef int vdi_gethdrs_f(VRT_CTX, VCL_BACKEND);
@@ -541,14 +661,13 @@ struct vdi_methods {
 	vdi_list_f			*list;
 };
 
-struct vcldir;
-
 struct director {
 	unsigned			magic;
 #define DIRECTOR_MAGIC			0x3336351d
 	void				*priv;
 	char				*vcl_name;
 	struct vcldir			*vdir;
+	struct lock			*mtx;
 };
 
 VCL_BOOL VRT_Healthy(VRT_CTX, VCL_BACKEND, VCL_TIME *);
@@ -559,46 +678,36 @@ void VRT_DisableDirector(VCL_BACKEND);
 VCL_BACKEND VRT_LookupDirector(VRT_CTX, VCL_STRING);
 void VRT_DelDirector(VCL_BACKEND *);
 
-/* Suckaddr related */
-int VRT_VSA_GetPtr(VRT_CTX, VCL_IP sua, const unsigned char ** dst);
-/* transitional interface */
-void VRT_Format_Proxy(struct vsb *, VCL_INT, VCL_IP, VCL_IP, VCL_STRING);
+/***********************************************************************
+ * vmod_priv related
+ */
+typedef void vmod_priv_fini_f(VRT_CTX, void *);
 
-typedef int vmod_event_f(VRT_CTX, struct vmod_priv *, enum vcl_event_e);
-
-typedef void vmod_priv_free_f(void *);
-struct vmod_priv {
-	void			*priv;
-	long			len;
-	vmod_priv_free_f	*free;
+struct vmod_priv_methods {
+	unsigned			magic;
+#define VMOD_PRIV_METHODS_MAGIC	0xcea5ff99
+	const char			*type;
+	vmod_priv_fini_f		*fini;
 };
 
-void VRT_priv_fini(const struct vmod_priv *p);
+struct vmod_priv {
+	void				*priv;
+	long				len;
+	const struct vmod_priv_methods	*methods;
+};
+
+void VRT_priv_fini(VRT_CTX, const struct vmod_priv *p);
 struct vmod_priv *VRT_priv_task(VRT_CTX, const void *vmod_id);
+struct vmod_priv *VRT_priv_task_get(VRT_CTX, const void *vmod_id);
 struct vmod_priv *VRT_priv_top(VRT_CTX, const void *vmod_id);
+struct vmod_priv *VRT_priv_top_get(VRT_CTX, const void *vmod_id);
 
-/* Stevedore related functions */
-int VRT_Stv(const char *nm);
-VCL_STEVEDORE VRT_stevedore(const char *nm);
+/***********************************************************************
+ * VSM and VSC
+ */
 
-/* Convert things to string */
-
-int VRT_CompareStrands(VCL_STRANDS a, VCL_STRANDS b);
-VCL_BOOL VRT_Strands2Bool(VCL_STRANDS);
-uint32_t VRT_HashStrands32(VCL_STRANDS);
-char *VRT_Strands(char *, size_t, VCL_STRANDS);
-VCL_STRING VRT_StrandsWS(struct ws *, const char *, VCL_STRANDS);
-VCL_STRING VRT_CollectStrands(VRT_CTX, VCL_STRANDS);
-VCL_STRING VRT_UpperLowerStrands(VRT_CTX, VCL_STRANDS s, int up);
-
-VCL_STRING VRT_BACKEND_string(VCL_BACKEND);
-VCL_STRING VRT_BOOL_string(VCL_BOOL);
-VCL_STRING VRT_CollectString(VRT_CTX, const char *p, ...);
-VCL_STRING VRT_INT_string(VRT_CTX, VCL_INT);
-VCL_STRING VRT_IP_string(VRT_CTX, VCL_IP);
-VCL_STRING VRT_REAL_string(VRT_CTX, VCL_REAL);
-VCL_STRING VRT_STEVEDORE_string(VCL_STEVEDORE);
-VCL_STRING VRT_TIME_string(VRT_CTX, VCL_TIME);
+struct vsmw_cluster *VRT_VSM_Cluster_New(VRT_CTX, size_t);
+void VRT_VSM_Cluster_Destroy(VRT_CTX, struct vsmw_cluster **);
 
 #ifdef va_start	// XXX: hackish
 void *VRT_VSC_Alloc(struct vsmw_cluster *, struct vsc_seg **,
@@ -609,7 +718,7 @@ void VRT_VSC_Hide(const struct vsc_seg *);
 void VRT_VSC_Reveal(const struct vsc_seg *);
 size_t VRT_VSC_Overhead(size_t);
 
-/*
+/***********************************************************************
  * API to restrict the VCL in various ways
  */
 
@@ -619,3 +728,10 @@ void VRT_VCL_Allow_Cold(struct vclref **);
 
 struct vclref * VRT_VCL_Prevent_Discard(VRT_CTX, const char *);
 void VRT_VCL_Allow_Discard(struct vclref **);
+
+/***********************************************************************
+ * Deprecated interfaces, do not use, they will disappear at some point.
+ */
+
+VCL_VOID VRT_synth_page(VRT_CTX, VCL_STRANDS);
+int VRT_Stv(const char *nm);
